#include "freertos/FreeRTOS.h"
#include "freertos/task.h"    
#include "esp_wifi.h"         
#include "esp_system.h"       
#include "esp_event.h"        
#include "esp_event_loop.h"   
#include "nvs_flash.h"        

#include "soc/gpio_struct.h"
#include "driver/gpio.h"    
#include "driver/adc.h"     
#include "esp_adc_cal.h"    
#include "esp_intr_alloc.h" 
#include "driver/dac.h"     
#include "driver/ledc.h"    

#include "eid430.h"         
#include "Gestion_Balance.h"
#include "HX711.h"          
#include "rom/gpio.h"       

#define LEDC_HS_TIMER          LEDC_TIMER_0
#define LEDC_HS_MODE           LEDC_HIGH_SPEED_MODE
#define LEDC_HS_CH0_GPIO       (25)
#define LEDC_HS_CH0_CHANNEL    LEDC_CHANNEL_0

#define TARE      GPIO_NUM_35 
#define  ESP_INTR_FLAG_DEFAUT   0
#define GPIO_BP_PIN_SEL     (1ULL<<TARE)

#define LEDC_TEST_FADE_TIME    (100)

ledc_timer_config_t ledc_timer = {
    .duty_resolution = LEDC_TIMER_8_BIT, // resolution of PWM duty
    .freq_hz = 1000,                      // frequency of PWM signal
    .speed_mode = LEDC_HS_MODE,           // timer mode
    .timer_num = LEDC_HS_TIMER            // timer index
};

const ledc_channel_config_t ledc_conf =
{
    .gpio_num = LEDC_HS_CH0_GPIO,           
    .speed_mode = LEDC_HS_MODE,
    .channel = LEDC_HS_CH0_CHANNEL,         
    .intr_type = LEDC_INTR_DISABLE,    
    .timer_sel = LEDC_HS_TIMER,         
    .duty = 0                                           
};

static xQueueHandle bp_evt_queue = NULL;
bool send_balance; 
uint32_t data_balance;
uint32_t cpt_bp, BP;
int etat_bp;

static void taskBalance(void *p)
{
    int mass;
    int rc_led = 0;
    float m_kg = 0;      
    uint32_t ref_val = 0;
    uint32_t moy = 0;    
    bool _tar = true;

    while(1)
    {
        vTaskDelay(1000 / portTICK_RATE_MS); 
        moy = lire_moyenne();
        if (etat_bp == 1 || _tar == true)
        {
            ref_val = moy;
            _tar = false;
            etat_bp = 0;
            printf("Tare en cours \r\n ref_val %x\r\n",ref_val);
        }
        else
        {
            if(moy > ref_val)
                mass=moy-ref_val;
            else
                mass=0;
            m_kg=((float)mass)/435555;  // rapport de division à modifier pour procédure d'étalonnage

            rc_led = (mass & 0x3FC000)>>14;
            rc_led = rc_led ^0xFF;
            ledc_set_fade_with_time(ledc_conf.speed_mode,ledc_conf.channel, rc_led, LEDC_TEST_FADE_TIME);
            ledc_fade_start(ledc_conf.speed_mode,ledc_conf.channel, LEDC_FADE_NO_WAIT);
            printf("RC = %x",rc_led);

            if (send_balance==false)
            {
                data_balance=(int)(1000*m_kg);
                send_balance=true;
            }
            printf(" m=%x conv=%x ref=%x m_kg=%1.5f\n",mass,moy,ref_val,m_kg);
        }
    }
}

static void IRAM_ATTR bp_isr_handler(void *arg)
{
    uint32_t bp_num=(uint32_t)arg;
    xQueueSendFromISR(bp_evt_queue,&bp_num,NULL);
}

static void taskBP(void *arg) // interruption
{
    uint32_t io_num;
    while(1)
    {
        if (xQueueReceive(bp_evt_queue,&io_num,portMAX_DELAY))
        {
            BP = gpio_get_level(io_num);

            if(BP == 0 && etat_bp == 0)
            {
                etat_bp = 1;
            }
        }
    }
}


void  InitGestionBalance()
{
    send_balance = false;
    data_balance = 0;
    etat_bp = 0;
    gpio_config_t io_conf;

    printf("init TARE");

    gpio_pad_select_gpio(TARE);     // bouton TARE
    gpio_set_direction(TARE,GPIO_MODE_DEF_INPUT);

    printf("init Timer");

    ledc_timer_config(&ledc_timer);

    printf("init PWM");

    ledc_channel_config(&ledc_conf);
    ledc_fade_func_install(0);

    printf("init port interrupt\n");
    
    io_conf.pin_bit_mask=GPIO_BP_PIN_SEL;
    io_conf.mode=GPIO_MODE_INPUT;
    io_conf.intr_type=GPIO_INTR_NEGEDGE;
    gpio_config(&io_conf);  

    //create a queue to handle gpio event from isr
    bp_evt_queue = xQueueCreate(10, sizeof(uint32_t));
    //install gpio isr service
    gpio_install_isr_service(ESP_INTR_FLAG_DEFAUT);
    //hook isr handler for specific gpio pin
    gpio_isr_handler_add(TARE, bp_isr_handler, (void*) (TARE));

    //remove isr handler for gpio number.
    gpio_isr_handler_remove(TARE);
    //hook isr handler for specific gpio pin again
    gpio_isr_handler_add(TARE,bp_isr_handler, (void*) TARE);  

    Init_HX711();

    //Démarrage de la tache
    xTaskCreate(taskBalance,"task_balance",2048,NULL,5,NULL);
    xTaskCreate(taskBP,"task_bp",2048,NULL,10,NULL);
}